package tech.riemann.ims.controller.platform.material;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import tech.riemann.ims.dto.request.ApplyInfo;
import tech.riemann.ims.dto.response.ApplyDTO;
import tech.riemann.ims.entity.material.ApplyDetail;
import tech.riemann.ims.entity.material.ApplyForm;
import tech.riemann.ims.entity.material.Material;
import tech.riemann.ims.entity.material.MaterialStockDetail;
import tech.riemann.ims.enums.*;
import tech.riemann.ims.service.material.IApplyDetailService;
import tech.riemann.ims.service.material.IApplyFormService;
import tech.riemann.ims.service.material.IMaterialService;
import tech.riemann.ims.service.material.IMaterialStockDetailService;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author mayong
 * @since 2024/11/28 15:29
 */
@RestController
@Tag(name = "Apply", description = "入库/出库 申请单管理")
@RequiredArgsConstructor
public class ApplyFormController {

    private final IApplyFormService applyFormService;
    private final IApplyDetailService applyDetailService;
    private final IMaterialStockDetailService materialStockDetailService;
    private final IMaterialService materialService;

    @GetMapping("applies")
    @Operation(summary = "分页查询申请列表")
    public IPage<ApplyDTO> searchPage(
            @Parameter(description = "页码") @RequestParam(name = "page", required = false, defaultValue = "1") int page,
            @Parameter(description = "页面大小") @RequestParam(name = "size", required = false, defaultValue = "10") int size,
            @Parameter(description = "申请类型(1入库 3出库 4盘点)") @RequestParam(name = "applyType") Integer applyType,
            @Parameter(description = "物料类型") @RequestParam(name = "type", required = false) String type,
            @Parameter(description = "物料编码") @RequestParam(name = "code", required = false) String code,
            @Parameter(description = "物料名称") @RequestParam(name = "name", required = false) String name) {
        return applyFormService.search(page, size, applyType, type, code, name);
    }

    @GetMapping("audit-applies")
    @Operation(summary = "分页查询盘点列表")
    public IPage<ApplyForm> searchAuditPage(
            @Parameter(description = "页码") @RequestParam(name = "page", required = false, defaultValue = "1") int page,
            @Parameter(description = "页面大小") @RequestParam(name = "size", required = false, defaultValue = "10") int size,
            @Parameter(description = "盘点类型") @RequestParam(name = "auditType", required = false) AuditTypeEnum auditType,
            @Parameter(description = "盘点人") @RequestParam(name = "taker", required = false) String taker,
            @Parameter(description = "创建日期") @RequestParam(name = "createDate", required = false) LocalDateTime createDate,
            @Parameter(description = "审核状态") @RequestParam(name = "reviewResult", required = false) ReviewResultEnum reviewResult) {
        return applyFormService.page(Page.of(page, size), Wrappers.<ApplyForm>lambdaQuery()
                .eq(auditType != null, ApplyForm::getAuditType, auditType)
                .eq(taker != null, ApplyForm::getTaker, taker)
                .eq(createDate != null, ApplyForm::getCreatedTime, createDate)
                .eq(reviewResult != null, ApplyForm::getReviewResult, reviewResult)
                .orderByDesc(ApplyForm::getCreatedTime)
        );
    }

    @GetMapping("apply/{applyId}")
    @Operation(summary = "查询申请单详情")
    public ApplyInfo detail(@Parameter(description = "类型") @PathVariable(name = "applyId") Integer applyId) {
        List<ApplyDetail> list = applyDetailService.list(Wrappers.<ApplyDetail>lambdaQuery().eq(ApplyDetail::getApplyId, applyId));
        return ApplyInfo.builder()
                .applyForm(applyFormService.getById(applyId))
                .applyDetails(list)
                .build();
    }

    @PutMapping("apply")
    @Operation(summary = "提交出入库申请单")
    @Transactional(rollbackFor = Exception.class)
    public void saveApply(@Validated @Parameter(description = "申请单") @RequestBody ApplyInfo applyInfo) {
        // 如果是采购入库 就新增数据，否则只改状态
        ApplyTypeEnum applyType = applyInfo.getApplyForm().getType();

        // 校验物料数量
        Map<Long, Integer> confirmMap = applyInfo.getApplyDetails().stream()
                .filter(item -> item.getAssignRule() != AssignRuleEnum.LOW_VALUE)
                .collect(Collectors.toMap(ApplyDetail::getMaterialId, ApplyDetail::getConfirmQuantity));
        Map<Long, Long> dataMap = applyInfo.getDetailList().stream()
                .collect(Collectors.groupingBy(MaterialStockDetail::getMaterialId, Collectors.counting()));
        confirmMap.forEach((k, v) -> {
            Long l = dataMap.get(k);
            if (l == null || l.intValue() != v) {
                throw new IllegalArgumentException("[" + k + "]物料数量不匹配");
            }
        });
        // 新增申请单
        applyFormService.save(applyInfo.getApplyForm());
        for (ApplyDetail applyDetail : applyInfo.getApplyDetails()) {
            applyDetail.setApplyId(applyInfo.getApplyForm().getId());
        }
        applyDetailService.saveBatch(applyInfo.getApplyDetails());

        if (applyType == ApplyTypeEnum.PURCHASE_RECEIPT) {
            // 新增物料
            if(!applyInfo.getDetailList().isEmpty()){
                applyInfo.getDetailList().forEach(detail -> {
                    detail.setApplyId(applyInfo.getApplyForm().getId());
                    detail.setStatus(StockStatusEnum.IN);
                });
                materialStockDetailService.saveBatch(applyInfo.getDetailList());
            }
            //记录物料库存数据
            applyInfo.getApplyDetails().forEach(detail ->
                    materialService.update(Wrappers.<Material>lambdaUpdate()
                            .eq(Material::getId, detail.getMaterialId())
                            .setSql(detail.getAssignRule() == AssignRuleEnum.HIGH_VALUE,
                                    "m_stock = m_stock + " + detail.getConfirmQuantity())
                            .setSql(detail.getAssignRule() == AssignRuleEnum.LOW_VALUE,
                                    "m_stock = m_stock + " + detail.getQuantity())
                    ));
        } else if (applyType == ApplyTypeEnum.LOAN_OUT) {
            //修改物料状态
            if(!applyInfo.getDetailList().isEmpty()){
                applyInfo.getDetailList().forEach(detail -> materialStockDetailService.update(Wrappers.<MaterialStockDetail>lambdaUpdate()
                        .set(MaterialStockDetail::getStatus, StockStatusEnum.OUT)
                        .eq(MaterialStockDetail::getBarcode, detail.getBarcode())));
            }
            //修改物料库存数据
            applyInfo.getApplyDetails().forEach(detail -> materialService.update(Wrappers.<Material>lambdaUpdate()
                    .eq(Material::getId, detail.getMaterialId())
                    .setSql(detail.getAssignRule() == AssignRuleEnum.HIGH_VALUE,
                            "m_stock = m_stock - " + detail.getConfirmQuantity())
                    .setSql(detail.getAssignRule() == AssignRuleEnum.LOW_VALUE,
                            "m_stock = m_stock - " + detail.getQuantity())
            ));
        } else if (applyType == ApplyTypeEnum.RETURN_RECEIPT) {
            //修改物料状态
            if(!applyInfo.getDetailList().isEmpty()){
                applyInfo.getDetailList().forEach(detail -> materialStockDetailService.update(Wrappers.<MaterialStockDetail>lambdaUpdate()
                        .set(MaterialStockDetail::getStatus, StockStatusEnum.IN)
                        .eq(MaterialStockDetail::getBarcode, detail.getBarcode())));
            }
            //修改物料库存数据
            applyInfo.getApplyDetails().forEach(detail -> materialService.update(Wrappers.<Material>lambdaUpdate()
                    .eq(Material::getId, detail.getMaterialId())
                    .setSql(detail.getAssignRule() == AssignRuleEnum.HIGH_VALUE,
                            "m_stock = m_stock + " + detail.getConfirmQuantity())
                    .setSql(detail.getAssignRule() == AssignRuleEnum.LOW_VALUE,
                            "m_stock = m_stock + " + detail.getQuantity())
            ));
        }
    }


}
